local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Utilities = import('/lua/utilities.lua');
local Mav
local CapArea
local PlayerMapR = {
Score = {}
}
local PreviousOwnerIndex = -1
local CatFilter
local DefencesCleared = false
local FirstTaker = true
local NeutralCivIndex
local ChatTo = import('/lua/lazyvar.lua').Create()

local MavorCaptureProgress = 0
local MavorMethodType = 'T3PGEN'
local ContestRule = 'GREATERSCORE'
local CivBrain
local DefencesList = {}
local DefencesListS = {}
-- T3PGEN: Build at least 2 T3 Pgen near mavor to capture.
-- BMH: Biggest Mass Holder (Far more Agressive gameplay)
-- COMS: Support coms and ACUs, forces them to come out to play. ACUs are worth 100 SACUs as score.

-- Capture Rules: After Guardians are defeated, deploy at least 1 T3 Power Generator near Mavor to initiate capture (With 1 T3 Pgen it takes 4 minutes). The more are deployed, the faster it will be captured.
-- All Power generators of a Player must be destroyed to break his control of the Mavor
-- If all PGens are destroyed, the Mavor will go back to the default state.

--Contest Rules:
--KICKKING : Current winner must have its score driven below threshold to allow capture. (AOE2 Uses this I believe)
--GREATERSCORE : Simply have a greater score than the Winner to get capture.
--NOCONTEST : Aka Kill the King to allow another capture.

function OnPopulate()
	--Standard start game stuff
    ScenarioUtils.InitializeArmies()
    ScenarioFramework.SetPlayableArea('AREA_1' , false)
	
	--Initialize the Mavor
	ScenarioInfo.Mavor = ScenarioUtils.CreateArmyUnit( 'NEUTRAL_CIVILIAN', 'MavorMid' )
	ScenarioInfo.Mavor:SetCustomName('Mavor (Koth)')
    ScenarioInfo.Mavor:SetCanTakeDamage(false)
	ScenarioInfo.Mavor:SetMaxHealth(99999)
	ScenarioInfo.Mavor:SetHealth(ScenarioInfo.Mavor,99999)
	ScenarioInfo.Mavor:SetRegenRate(99999)
    ScenarioInfo.Mavor:SetCanBeKilled(false)
    ScenarioInfo.Mavor:SetReclaimable(false)
	ScenarioInfo.Mavor:SetCapturable(false)
	#ScenarioFramework.CreateTimerTrigger(MavorReset,2)
	#ScenarioFramework.CreateTimerTrigger(MavorGiveToTeam,20)
	
	NeutralCivIndex = ScenarioInfo.Mavor:GetArmy()
	#CivBrain = ScenarioFramework:GetArmyBrain()
	
	--Create our Guardians
	
	ScenarioInfo.MavorGuardians = ScenarioUtils.CreateArmyGroup( 'ARMY_17','Guardians' )
	ScenarioInfo.Defences = ScenarioUtils.CreateArmyGroup( 'ARMY_17','Defences' )
	ScenarioInfo.DefencesSecondary = ScenarioUtils.CreateArmyGroup( 'NEUTRAL_CIVILIAN','DefencesSecondary' )
	for i , G in ScenarioInfo.MavorGuardians do
		G:SetCustomName('Ravager (Mavor Override)')
	end
	
	--Capture vars
	local NumArmies = 1
	
	#LOG('ArmyList: ' .. ListArmies())
	for k, army in ListArmies() do
		if not ArmyIsCivilian(NumArmies) then
			PlayerMapR.Score[NumArmies] = 0
		end
		NumArmies = NumArmies +1
	end
	
	CapArea = ScenarioUtils.AreaToRect('MAVOR_CAPTURE')	
	ScenarioFramework.CreateTimerTrigger(DefencesClearedCheck,10)	
end

--Ripped from Survival 5Th Dimension
function FrameSay(Message,Color,Time)
	PrintText(Message, 20, Color, Time, 'center') ;	
end



function MavorReset()
	#ScenarioInfo.Mavor:SetCanTakeDamage(false)
	ScenarioInfo.Mavor:SetMaxHealth(99999)
	ScenarioInfo.Mavor:SetHealth(ScenarioInfo.Mavor,99999)
	#ScenarioInfo.Mavor:SetRegenRate(99999)
    ScenarioInfo.Mavor:SetCanBeKilled(false)
    ScenarioInfo.Mavor:SetReclaimable(false)
	ScenarioFramework.CreateTimerTrigger(MavorReset,2)
end

function DefencesClearedCheck()
	local GLeft = 0
	for	k , GI in ScenarioInfo.MavorGuardians do
		if not GI:IsDead() then
		GLeft = GLeft + 1
		end
	end
	#LOG('Guardians Left: ' .. GLeft)
	if GLeft > 0 then --There are still guardians alive. Repeat the check a little later
		ScenarioFramework.CreateTimerTrigger(DefencesClearedCheck,5)
	else --Yay, no more Guardians, now we will see who gets to own the Mavor now!
		FrameSay('WARNING: MAVOR AUTOMATIC OVERRIDES ARE COMPROMISED!','ffffff01',4)
		DelayFrameSay('DEPLOY AT LEAST 4 T3 POWER GENERATORS NEAR MAIN GUN TO ENGAGE MANUAL CONTROLS.','ffffff01',8,5)
		CheckMavorCaptureStatus()
	end
end

function DelayFrameSay(Message,Color,Time,Delay)
	WaitSeconds( Delay )
	FrameSay(Message,Color,Time)
end

function CheckMavorCaptureStatus()
	local unarea
	local UCount = 0
	local WinnerID = -1
	local PrvScore = 0
	if MavorMethodType == 'T3PGEN' then
	PrvScore = 3.9
	end
	unarea = GetUnitsInRect(CapArea)	
	
	--Reset these values so we may get a new winner
	for k, ply in PlayerMapR.Score do
		PlayerMapR.Score[k] = 0
	end
	
	MavorCaptureScoreMethod(MavorMethodType,unarea)
	
	--We find out who won the contest
	for k, ply in PlayerMapR.Score do	
		if ply > PrvScore then			
			WinnerID = k
			PrvScore = ply
		end
	end 
	
	--We decide upon the new winner in what to do.
	if WinnerID ~= PreviousOwnerIndex then --We check if the winner is the previous winner so we don't give the Mavor to them twice. 
		if (WinnerID >0) then 
			if PlayerMapR.Score[PreviousOwnerIndex] < PlayerMapR.Score[WinnerID] then --We check if the new winner has a superior score, this is so the previous winner isn't dethroned despite a tie.
				PreviousOwnerIndex = WinnerID
				MavorGiveToTeam(WinnerID)
			end
		else
			#LOG('Give mavor to civies')
			PreviousOwnerIndex = -1
			MavorGiveToTeam(NeutralCivIndex) --Abandoned or something. So we give it back to the Civilians
		end
		
	end
	ScenarioFramework.CreateTimerTrigger(CheckMavorCaptureStatus,3)
end


function MavorCaptureScoreMethod(Method,unarea)
	if Method == 'T3PGEN' then
		if unarea then
            for k,unit in unarea do				
				if not ArmyIsCivilian(unit:GetArmy()) then
					if not unit:IsDead() and not unit:IsBeingBuilt() then
					--Filter our units so flying units and walls do not qualify.
						if EntityCategoryContains(categories.TECH3,unit) and EntityCategoryContains(categories.ENERGYPRODUCTION,unit) and EntityCategoryContains(categories.STRUCTURE,unit) then
											-- Add the mass values from all the units on site.
						PlayerMapR.Score[unit:GetArmy()] = PlayerMapR.Score[unit:GetArmy()] + 1
						
						end						
					end
                end
            end
		end
	elseif Method == 'BMH' then
		if unarea then
            for k,unit in unarea do				
				if not ArmyIsCivilian(unit:GetArmy()) then
					if not unit:IsDead() and not unit:IsBeingBuilt() then
					--Filter our units so flying units and walls do not qualify.
						if not EntityCategoryContains(categories.WALL,unit) and not EntityCategoryContains(categories.AIR,unit) and unit:GetBlueprint().Economy.BuildCostMass < 100000 then
											-- Add the mass values from all the units on site.
						PlayerMapR.Score[unit:GetArmy()] = PlayerMapR.Score[unit:GetArmy()] + unit:GetBlueprint().Economy.BuildCostMass			
						end						
					end
                end
            end
		end
	elseif Method == 'COMS' then
	
	end
end

function MavorCaptureProcess()
	
end

function MavorGiveToTeam(OwnerID)
	#LOG('Mavor given to team')
	if (FirstTaker) then
	FrameSay('GUARDIAN\'S TRAIL HAS BEEN ENGAGED!','ffcd0000',7)
	--Title drop for good measure. In other words, everyone get to mid ASAP or DIE
	Mav = ScenarioFramework.GiveUnitToArmy(ScenarioInfo.Mavor, OwnerID )
	
	for k,v in ScenarioInfo.Defences do
        if not v:IsDead() then
            DefencesList[k] = ScenarioFramework.GiveUnitToArmy( v, OwnerID )
        end
    end	
	
	for k,v in ScenarioInfo.DefencesSecondary do
        if not v:IsDead() then
            DefencesListS[k] = ScenarioFramework.GiveUnitToArmy( v, OwnerID )
        end
    end	
	FirstTaker = false
	else	
	
	if (NeutralCivIndex ~= OwnerID) then -- Failsafe just in case it tries to be gifted to the same user twice	
	FrameSay('GUARDIAN\'S TRAIL HAS BEEN REENGAGED!','ffcd0000',7)	
	end
	Mav = ScenarioFramework.GiveUnitToArmy( Mav, OwnerID )	
	for k,v in DefencesList do
        if not v:IsDead() then
            DefencesList[k] = ScenarioFramework.GiveUnitToArmy( v, OwnerID )
        end
    end
	
	for k,v in DefencesListS do
        if not v:IsDead() then
            DefencesListS[k] = ScenarioFramework.GiveUnitToArmy( v, OwnerID )
        end
    end	
	end	
	Mav:SetMaxHealth(99999)
	Mav:SetHealth(ScenarioInfo.Mavor,99999)	
	Mav:SetCanBeKilled(false)
    Mav:SetReclaimable(false)
	Mav:SetCanTakeDamage(false)
	Mav:SetCapturable(false)
end

function OnStart(scenario)
end
